@@ -40,7 +40,7 @@
 #include <gst/gst.h>
 #include <gst/pbutils/pbutils.h>
 
-#include <libgupnp-dlna/gupnp-dlna-load.h>
+#include <libgupnp-dlna/profile-loading.h>
 #include <libgupnp-dlna/gupnp-dlna-profile.h>
 #include <libgupnp-dlna/gupnp-dlna-discoverer.h>
 #include <libgupnp-dlna/gupnp-dlna-information.h>
@@ -452,7 +452,8 @@ main (int argc, char **argv)
 
         GOptionContext *ctx;
 
-        g_thread_init(NULL);
+        if (!g_thread_supported ())
+                g_thread_init(NULL);
 
         ctx = g_option_context_new (" - program to extract DLNA and related metadata");
         g_option_context_add_main_entries (ctx, options, NULL);
